#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>

#define P_ERROR	-1						// to check for errors when dealing with the port

int openPort(void);
void closePort(int fd);
void writeData(int fd, int nbytes);
void readData(int fd, int nbytes);

struct termios options;

unsigned char sbuf[4];						// Stores data to be written
unsigned char sbufr[4];						// Stores data to be read

int main(int argc, char *argv[])
{
int fd;								// file descriptor of open port

	fd = openPort();						// Open port to USBi2C	
	
	if(fd > P_ERROR)						// Only do the folloing if there has not been an error opening the port
	{
		
		sbuf[0] = 0x5a;						// USBI2C command for single byte address read write
		sbuf[1] = 0x10;
		sbuf[2] = 0x0e;
		sbuf[3] = 0x00;
		
		writeData(fd, 4);					// Write this data to the USBI2C
	
		usleep(15000);		

		readData(fd, 1);					// Read back the 4 bytes.

		closePort(fd);						// Close opened port
		
		printf("response_0: %u\n", sbufr[0]);		// Display data to the screen
		printf("response_1: %u\n", sbufr[1]);
		printf("response_2: %u\n", sbufr[2]);
		printf("response_3: %u\n", sbufr[3]);
	}
	return 0;
}


int openPort(void)
{
int fd;										// File descriptor for the port

	fd = open("/dev/dtyU0", O_RDWR );				// Open port for read and write not making it a controlling terminal
	if (fd == P_ERROR)
	{
		perror("openPort: Unable to open /dev/ttyU0 - ");		// If open() returns an error
	} 
	else
	{
		fcntl(fd, F_SETFL, 0);						// Get the current options for the port
		tcgetattr(fd, &options);
		
		cfsetispeed(&options, B9600);				// Set the baud rates to 9600

		options.c_cflag |= (CLOCAL | CREAD);		// Enable the receiver and set local mode
		options.c_cflag &= ~PARENB;					// No parity bit

		options.c_cflag &= ~CSTOPB;					// Set 2 stop bits

		options.c_cflag &= ~CSIZE;					// Set the character size
		options.c_cflag |= CS8;

		tcsetattr(fd, TCSANOW, &options);				// Set the new options for the port
	}
	return (fd);
}


void closePort(int fd)
{	
	if(close(fd) == P_ERROR)						// Close the port if it returns an error then display an error report
	{	
		perror("closePort: Unable to close /dev/dtyU0 - ");
	}
}


void writeData(int fd, int nbytes)
{
int bytes;
	
	bytes = write(fd, sbuf, nbytes);					// Write nbytes of data from wbuf
	if(bytes == P_ERROR)							// If write returns an error (-1)
	{
		perror("writeData: Error while trying to write data - ");	
	}
	else if(bytes != nbytes)
	{
		printf("only %u bytes written out of %u requested\n", bytes, nbytes);	
	}

}


void readData(int fd, int nbytes)
{
int bytes;

	bytes = read(fd, sbufr, nbytes);						// Read nbytes of data into rbuf
	if(bytes == P_ERROR)							// If read returns an error (-1)
	{
		perror("readData: Error while trying to read data - ");
	}
	else if(bytes != nbytes)
	{
		printf("Only %u bytes read out of %u requested\n", bytes, nbytes);
	}
}
